#!/usr/bin/python
#
#    conv_cmf_obj.py : Conversion de fichier binaire Cal3D CMF en OBJ
#    Copyright (C) 2008  Olivier Boyaval (Etory)
#
#    This program is free software: you can redistribute it and/or modify
#===============================================================================
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
# 
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
# 
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#===============================================================================
#

import struct, sys, os.path

#
#ecrit les vertex dans le fichier obj
#
def write_vertices(fiObj, fiCmf, vertices, maps, springs):
    #boucle des vertex (point 3d)
    boucle_vertices=vertices
    while boucle_vertices:   
        #print 'vertice : %i' % (vertices-boucle_vertices+1)
        vx=struct.unpack("<f", fiCmf.read(4))[0]
        vz=-struct.unpack("<f", fiCmf.read(4))[0]
        vy=struct.unpack("<f", fiCmf.read(4))[0]
        fiObj.write('v %f %f %f\n' % (vx, vy, vz))
        nx=struct.unpack("<f", fiCmf.read(4))[0]
        nz=-struct.unpack("<f", fiCmf.read(4))[0]
        ny=struct.unpack("<f", fiCmf.read(4))[0]
        fiObj.write('vn %f %f %f\n' % (nx, ny, nz))
        # jete les donnees non utilisees (8 octets)
        poubelle=fiCmf.read(8)
        # recupere l'UV mapping
        # attention ne fonctionne pas si plusieurs uvmapping l'un sur l'autre
        boucle_maps=maps
        while boucle_maps:
            mu=struct.unpack("<f", fiCmf.read(4))[0]
            mv=struct.unpack("<f", fiCmf.read(4))[0]
            fiObj.write('vt %f %f %f\n' % (mu, mv, 0.0))
            boucle_maps=boucle_maps-1
        # Jete les donnees d'attachement au squelette puisque le squelette
        # n'est pas recupere 
        boucle_influence=struct.unpack("<i", fiCmf.read(4))[0]
        while boucle_influence:
            poubelle=fiCmf.read(8)
            boucle_influence=boucle_influence-1
        # Jete les 4 octets de spring si ils sont presents
        if springs>0:
            poubelle=fiCmf.read(4)
        #fin de la boucle sur les vertices
        boucle_vertices=boucle_vertices-1

#
# ecrit les faces dans le fichiers obj
#
def write_faces(fiObj, fiCmf, faces):
    boucle_faces=faces
    while boucle_faces:
        f1=struct.unpack("<i",fiCmf.read(4))[0]+1
        f2=struct.unpack("<i",fiCmf.read(4))[0]+1
        f3=struct.unpack("<i",fiCmf.read(4))[0]+1
        fiObj.write('f %i/%i %i/%i %i/%i\n' % (f1,f1,f2,f2,f3,f3))
        boucle_faces=boucle_faces-1

#
#ecrit le mesh dans le fichier obj
#
def write_mesh(fiObj, fiCmf, name):
    # Recuperation des parametres du mesh
    materialid=struct.unpack("<i",fiCmf.read(4))[0]
    vertices=struct.unpack("<i",fiCmf.read(4))[0]
    faces=struct.unpack("<i",fiCmf.read(4))[0]
    lod=struct.unpack("<i",fiCmf.read(4))[0]
    springs=struct.unpack("<i",fiCmf.read(4))[0]
    maps=struct.unpack("<i",fiCmf.read(4))[0]
    #debug :
    #print materialid,vertices,faces,lod,springs,maps
    
    # ecrit les vertices dans le fichier obbj
    write_vertices(fiObj, fiCmf, vertices, maps, springs)
    
    # jete la table des springs
    boucle_springs=springs
    while boucle_springs:
        poubelle=fiCmf.read(16)
        boucle_springs=boucle_springs-1
    
    # ecrit le nom de l'objet dans le fichier obj
    fiObj.write('g object_%s\n' % (name))
    
    # ecrit les faces dans le fichiers obj
    write_faces(fiObj, fiCmf, faces)


#
# Programme principal
#
if len(sys.argv) != 2:
    print "synthaxe : %s nom_fichier.cmf" % (sys.argv[0])
    print "synthaxe : python %s nom_fichier.cmf"  % (sys.argv[0])
    sys.exit(2)

name_file = os.path.splitext(sys.argv[1])[0]
name_object = os.path.split(name_file)[1]

#Ouvre le fichier CMF en mode binaire
print "ouverture de %s" % (sys.argv[1])
try:
    fiCmf = open(sys.argv[1], 'rb')
except:
    print "%s n'est pas un fichier lisible" % (sys.argv[1])
    sys.exit(2)

# Verifie l'entete du fichier
type_fichier = fiCmf.read(4)
if type_fichier != "CMF\0" :
    print "erreur le fichier %s n'est pas un fichier CAL3D CMF" % (sys.argv[1])
else:
    # Debug :
    #print "Ok le fichier est bien un fichier CAL3D CMF"
    version = struct.unpack("<i",fiCmf.read(4))[0]
    nb_submesh = struct.unpack('<i',fiCmf.read(4))[0]
    # Debug :
    #print version
    #print nb_submesh
    
    #
    #boucle du nombre de submesh
    boucle_mesh=nb_submesh
    while boucle_mesh:
        # definition du nom du fichier Obj et du nom du mesh 
        if nb_submesh!=1 :
            #il y a plusieurs meshes donc il faut indexe les noms
            nameFiObj = "%s_%i.obj" % (name_file, nb_submesh-boucle_mesh)
            nameMesh = "%s_%i" % (name_object, nb_submesh-boucle_mesh)
        else:
            #sinon il n'y a qu'un mesh
            nameFiObj = "%s.obj" % (name_file)
            nameMesh = name_object
        # creation du fichier obj
        print "creation du fichier %s pour y stocker le mesh %s" % (nameFiObj, nameMesh)
        try:
            fiObj = open(nameFiObj, 'w')
        except:
            print "Impossible d'ecrire dans le fichier %s" % (nameFiObj)
            sys.exit(2)
        #ecriture du mesh dans le fichier obj
        write_mesh(fiObj, fiCmf, nameMesh)
        #ferme le fichier obj
        fiObj.close
        #on passe au mesh suivant
        boucle_mesh=boucle_mesh-1

#
# on ferme le fichier
fiCmf.close()
print "conversion termine"


